"""Python Bitcoin Tool"""
try:
    from importlib.metadata import PackageNotFoundError, version  # type: ignore
except ImportError:  # pragma: no cover
    from importlib_metadata import PackageNotFoundError, version  # type: ignore

try:
    __version__ = version(__name__)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"

from bitcoin.bci import fetchtx, history, pushtx, unspent
from bitcoin.deterministic import (
    TESTNET_PRIVATE,
    bip32_ckd,
    bip32_deserialize,
    bip32_extract_key,
    bip32_master_key,
    bip32_privtopub,
    electrum_mpk,
    electrum_privkey,
    electrum_pubkey,
    electrum_stretch,
)
from bitcoin.main import (
    G,
    N,
    access,
    add,
    add_privkeys,
    add_pubkeys,
    b58check_to_hex,
    bin_hash160,
    bin_sha256,
    compress,
    count,
    decode_privkey,
    decode_pubkey,
    decompress,
    deterministic_generate_k,
    divide,
    ecdsa_raw_recover,
    ecdsa_raw_sign,
    ecdsa_recover,
    ecdsa_sign,
    ecdsa_verify_addr,
    encode_privkey,
    encode_pubkey,
    fast_add,
    fast_multiply,
    hash160,
    multiaccess,
    multiply,
    privkey_to_address,
    privkey_to_pubkey,
    privtoaddr,
    privtopub,
    pubkey_to_address,
    pubtoaddr,
    random_electrum_seed,
    random_key,
    sha256,
    slice,
    sum,
)
from bitcoin.py3specials import (
    bytes_to_hex_string,
    changebase,
    decode,
    encode,
    from_string_to_bytes,
    safe_from_hex,
)
from bitcoin.stealth import (
    basic_stealth_address_to_pubkeys,
    ephem_pubkey_from_tx_script,
    mk_stealth_metadata_script,
    mk_stealth_tx_outputs,
    pubkeys_to_basic_stealth_address,
    shared_secret_receiver,
    shared_secret_sender,
    uncover_pay_privkey,
    uncover_pay_pubkey_receiver,
    uncover_pay_pubkey_sender,
)
from bitcoin.transaction import (
    address_to_script,
    apply_multisignatures,
    deserialize,
    deserialize_script,
    ecdsa_tx_recover,
    ecdsa_tx_sign,
    ecdsa_tx_verify,
    mk_multisig_script,
    mksend,
    mktx,
    multisign,
    script_to_address,
    scriptaddr,
    serialize,
    serialize_script,
    sign,
    txhash,
    verify_tx_input,
)

__all__ = [
    "changebase",
    "bytes_to_hex_string",
    "safe_from_hex",
    "encode",
    "decode",
    "from_string_to_bytes",
    # main
    "add",
    "privkey_to_address",
    "multiply",
    "G",
    "add_privkeys",
    "add_pubkeys",
    "pubtoaddr",
    "privtopub",
    "b58check_to_hex",
    "sha256",
    "compress",
    "ecdsa_sign",
    "decompress",
    "divide",
    "N",
    "ecdsa_raw_sign",
    "ecdsa_raw_recover",
    "ecdsa_recover",
    "ecdsa_verify_addr",
    "fast_multiply",
    "fast_add",
    "encode_pubkey",
    "encode_privkey",
    "decode_pubkey",
    "decode_privkey",
    "deterministic_generate_k",
    "bin_sha256",
    "privtoaddr",
    "random_key",
    "bin_hash160",
    "hash160",
    "privkey_to_pubkey",
    "pubkey_to_address",
    "random_electrum_seed",
    "access",
    "multiaccess",
    "slice",
    "count",
    "sum",
    # deterministic
    "electrum_mpk",
    "electrum_privkey",
    "electrum_pubkey",
    "bip32_privtopub",
    "bip32_ckd",
    "bip32_master_key",
    "bip32_deserialize",
    "TESTNET_PRIVATE",
    "electrum_stretch",
    "bip32_extract_key",
    # transaction
    "ecdsa_tx_sign",
    "ecdsa_tx_verify",
    "ecdsa_tx_recover",
    "serialize",
    "deserialize",
    "serialize_script",
    "deserialize_script",
    "script_to_address",
    "address_to_script",
    "mktx",
    "mksend",
    "sign",
    "multisign",
    "apply_multisignatures",
    "scriptaddr",
    "mk_multisig_script",
    "verify_tx_input",
    "txhash",
    # stealth
    "basic_stealth_address_to_pubkeys",
    "pubkeys_to_basic_stealth_address",
    "shared_secret_sender",
    "shared_secret_receiver",
    "uncover_pay_pubkey_sender",
    "uncover_pay_pubkey_receiver",
    "uncover_pay_privkey",
    "mk_stealth_metadata_script",
    "mk_stealth_tx_outputs",
    "ephem_pubkey_from_tx_script",
    # bci
    "history",
    "unspent",
    "fetchtx",
    "pushtx",
]
